#!/usr/bin/perl
# -------------------------------------------------------------------
# SortMp3s.pl
# -----------
# This PERL script sorts MP3s into the appropriate directories based
# on the information contained within the ID3 tags. If no information
# is found in the tags, it places the file into an 'Unsorted'
# directory.
# 
# It also saves the mp3 information into a NySQL database for future 
# access. Once the information is stored into the database, other 
# applications can access the information. Other applications to 
# use the database will be forthcoming from me if this application
# is actually used.
# -------------------------------------------------------------------
# (c) 2001 Mike Patton
# RELEASED UNDER THE GPL
# -------------------------------------------------------------------
# This software is released under the GPL. If you do not understand
# what the GPL is, find it and read it BEFORE using this software.
# -------------------------------------------------------------------
# REQUIREMENTS:
# -------------
# 1. MySQL database
#
# 2. PERL 5 or higher
#
# 3. DBI Interface to MySQL database
#
# 4. MP3::Info PERL module
# -------------------------------------------------------------------
# INSTRUCTIONS:
# -------------
# Set all of the values at the beginning of the script to match your
# setup. Run the script. Watch the magic! 
# -------------------------------------------------------------------
# DISCLAIMER:
# -----------
# I do not take any responsibility for anything that happens to your
# system or you personally for using this script. USE THIS SCRIPT AT
# YOUR OWN RISK. If you do not agree with this, do not use this 
# script.
#
# I do not provide any support for this script. I wrote it for my 
# personal use, and thought it would be useful for others. If you
# need a a custom solution, feel free to contact me and we can work
# out payment arrangements for custom development work. I am a very
# busy programmer, therefore I can not do free custom programming 
# work or help everyone set this script up.
#
# However, if you find bugs within this script, please send me your
# bugfixes and I will integrate them into a future release of the
# script.
# ------------------------------------------------------------------

use MP3::Info;
use DBI;
use strict;

eval {
   
   # ------- Set up these values to match your system setup. -------
   
   # This is the hostname where your MySQL server resides. For most
   # setups, you can leave it as is.
   my $DB_HOST = '';

   # This is the port on $DB_HOST where your MySQL server resides.
   # For most setups, you can leave it as is.
   my $DB_PORT = '';

   # This is the database name to use to store the information. You
   # should create this database for use by this application. To 
   # create the tables required for the application, see the file
   # included in this application titled 'INSTALL'
   my $DB_NAME = 'MP3s';

   # This is the username to be used for connecting to $DB_NAME. 
   # This user must have full access to $DB_NAME
   my $DB_USER = 'MP3s';

   # This is the password for the user $DB_USER.
   my $DB_PASS = 'MP3s';

   # This is the directory containing your unsorted MP3s. All 
   # of the subdirectories of this directory will be scanned for 
   # MP3s.
   my $SOURCE = '/Archives/Music';
   
   # These are directories you do not want the script to parse. 
   # The directories must be quoted and comma separated. 
   # Subdirectories of these directories will be skipped as well.
   my @skipDirectories = ( 
   );
   
   # This is the destination directory, where the sorted MP3s 
   # will be stored after sorting. This directory MUST exist, and
   # the user running this script must have access to this
   # directory. Any required subdirectories will be created.
   my $DEST = '/Archives/SortedMusic';   
   
   # ------ You should not have to modify anything below this line ------ #

   # Create basic directories.
   if ( ! -d "$DEST/UnSorted" ) {
      `mkdir $DEST/UnSorted`;
   }
   
   # Make the database connection
   my $dsn = "DBI:mysql:database=$DB_NAME";
   $dsn .= ";host=$DB_HOST" if $DB_HOST;
   $dsn .= ";port=$DB_PORT" if $DB_PORT;
   my $dbh = DBI->connect( $dsn, $DB_USER, $DB_PASS );
   
   # Exit if the database connection was not successful.
   if ( ! $dbh ) {
      die;
   }
   
   # Now get a list of all the MP3s which need to be 
   # sorted.
   my @mp3Files = getFileList( $SOURCE );
   print "Processing MP3s in $SOURCE";
   my $okFiles = 0;
   foreach my $tag ( @mp3Files ) {
      my ( $newPath, $dbEntry ) = processMp3( $tag );       
      $dbEntry->{filename} = "$newPath/$tag->{filename}";
      $dbEntry->{orig_filename} = $tag->{fullPath};
      my $errLvl = saveInfo( $dbEntry );
      if ( ! $errLvl ) { copyFile( $tag->{fullPath}, $newPath ); print '.'; $okFiles++; }
   }

   my $total = @mp3Files;
   print "\nOF $total MP3s PROCESSED, $okFiles WERE IMPORTED. OTHERS WERE NOT IMPORTED DUE TO INVALID FILENAMES OR DUPLICATIONS.\n";
   $dbh->disconnect();
   exit;
   
   sub copyFile {
      my ( $old, $new ) = @_;
      if ( ! -d $new ) {
         my @dirArry = split(/\//, $new);
         my $dir = shift @dirArry;
         foreach ( @dirArry ) {
            $dir .= "/$_";
            if ( ! -d $dir ) {
               `mkdir $dir`;
            }
         }
      }
      `cp $old $new`;
      return;
   }
   
   sub saveInfo {
      my $dbInfo = shift;
      my $genre = getID( 'Genres', $dbInfo->{genre} );
      my $artist = getID( 'Artists', $dbInfo->{artist} );
      my $album = getID( 'Albums', $dbInfo->{album} );
      my $year = $dbh->quote( $dbInfo->{year} );
      my $tracknum = $dbh->quote( $dbInfo->{tracknum} );
      my $filename = $dbh->quote( $dbInfo->{filename} );
      my $filesize = (stat($dbInfo->{orig_filename}))[7];
      my $name = $dbh->quote( $dbInfo->{title} );
      my $comment = $dbh->quote( $dbInfo->{comment} );
      my $sth = $dbh->prepare("SELECT COUNT(*) FROM Songs WHERE Artist = $artist AND Name = $name");
      $sth->execute();
      my @tmpCount = $sth->fetchrow_array();
      if ( @tmpCount[0] ) {
         print "d";
         return 1;
      }
      my $sql = "INSERT INTO Songs ( Genre, Artist, Album, Year, TrackNum, FileName, FileSize, Name, Comment ) VALUES ( $genre, $artist, $album, $year, $tracknum, $filename, $filesize, $name, $comment )";
      my $sth = $dbh->prepare($sql);    
      $sth->execute();
      $sth->finish();
      return 0;
   }

   sub getID {
      my ( $table, $value ) = @_;
      my $Name = $dbh->quote( $value );
      my $sth = $dbh->prepare("SELECT id FROM $table WHERE Name = $Name");
      $sth->execute();
      my @res = $sth->fetchrow_array();
      $sth->finish();
      my $id = $res[0];
      if ( ! $id ) {
         my $sth = $dbh->prepare("INSERT INTO $table ( Name ) VALUES ( $Name )");
         $sth->execute();
         $id = $dbh->{'mysql_insertid'};
         $sth->finish();
      }
      return $id;      
   }
   
   sub processMp3 {
      my $tag = shift;
      my %dbEntry;
      my $newPath = $DEST;
      my $id3Tag = get_mp3tag($tag->{fullPath});

      # Check for the GENRE tag.
      my ( $pathElement, $dbName ) = checkTag( $id3Tag, 'GENRE' );
      $dbEntry{'genre'} = $dbName;
      $newPath .= $pathElement;

      # Check for the artist tag.
      my ( $pathElement, $dbName ) = checkTag( $id3Tag, 'ARTIST' );
      $dbEntry{'artist'} = $dbName;
      $newPath .= $pathElement;

      # Check for the album tag.
      my ( $pathElement, $dbName ) = checkTag( $id3Tag, 'ALBUM' );
      $dbEntry{'album'} = $dbName;
      $newPath .= $pathElement;

      my ( $pathElement, $dbName ) = checkTag( $id3Tag, 'TITLE' );
      $dbEntry{'title'} = $dbName;

      my ( $pathElement, $dbName ) = checkTag( $id3Tag, 'YEAR' );
      $dbEntry{'year'} = $dbName;

      my ( $pathElement, $dbName ) = checkTag( $id3Tag, 'COMMENT' );
      $dbEntry{'comment'} = $dbName;

      my ( $pathElement, $dbName ) = checkTag( $id3Tag, 'TRACKNUM' );
      $dbEntry{'tracknum'} = $dbName;

      # If the new path has not been changed, we will
      # copy these files into the unsorted path.
      if ( $newPath eq $DEST ) {
         $newPath = "$DEST/UnSorted";
      }
      return( $newPath, \%dbEntry );
   }

   sub checkTag {
      my ( $tag, $tagItem ) = @_;
      my $newPath;
      my $dbElement = 'UNKNOWN';
      if ( $tag->{$tagItem} ) {
         $tag->{$tagItem} =~ s/\s+/ /g;
         $tag->{$tagItem} =~ s/^\s+|\s+$//;
         if ( $tagItem ne 'GENRE' ) {
            $dbElement = $tag->{$tagItem};
         }
            
         $tag->{$tagItem} =~ s/\&/ and /g;
         $tag->{$tagItem} =~ s/\'/ /g;
         $tag->{$tagItem} =~ s/\,/ /g;
         $tag->{$tagItem} =~ s/\-/ /g;
         my @tagArray = split(/\s+/, $tag->{$tagItem});
         if ( $tagItem eq 'GENRE' ) {
            $dbElement = $tag->{$tagItem};
            for ( my $x = 0 ; $x < @tagArray; $x++ ) {
               $tagArray[$x] = lc( $tagArray[$x] );
               $tagArray[$x] = ucfirst($tagArray[$x]) if ( $tagArray[$x] ne 'and' and $tagArray[$x] ne 'or' ) or $x eq 0;
            }
         }
         $newPath .= "/" . join( '_', @tagArray );
         if ( $newPath eq '/Rock' ) {
            $newPath = '/Rock_and_Roll';
            $dbElement = 'Rock & Roll';
         } elsif ( $newPath eq '/Rock_and_Roll' ) {
            $dbElement = 'Rock & Roll';
         }
      }
      return($newPath, $dbElement);
   }
   
   sub getFileList {
      my $dir = shift;
      my @returnFiles;
      opendir( DIR, $dir ) || die "Could not open directory $dir\n";
      my @listings = readdir( DIR );
      closedir( DIR );
      foreach my $item ( @listings ) {
         next if $item eq '.' or $item eq '..';
         $item =~ s/\(/\\\(/g;
         $item =~ s/\)/\\\)/g;
         $item =~ s/\&/\\\&/g;
         $item =~ s/\'/\\\'/g;
         $item =~ s/\</\\\</g;
         $item =~ s/\>/\\\>/g;
         $item =~ s/\[/\\\[/g;
         $item =~ s/\]/\\\]/g;
         $item =~ s/\s/\\\ /g;
#         $item =~ s/-/\\\-/g;
         my $origItem = $item;
         $item = "$dir/$item";
         if ( -d $item ) {
            my $skip = 0;
            foreach ( @skipDirectories ) {
               $skip = 1 if $item eq $_;
            }
            push @returnFiles, getFileList( $item ) if not $skip;
         } elsif ( $item =~ /\.mp3$/i ) {
            my %item = ( 'filename' => $origItem,
                         'fullPath' => $item ); 
            push @returnFiles, \%item;
         }
      }
      return @returnFiles;
   }
};

print "Application Error: $@" if $@;
