#!/usr/bin/perl
# --------------------------------------------------------------------
# makeMp3List.pl
# --------------
# This PERL script will create a MP3 listing.
# --------------------------------------------------------------------
# -------------------------------------------------------------------
# (c) 2001 Mike Patton
# RELEASED UNDER THE GPL
# -------------------------------------------------------------------
# This software is released under the GPL. If you do not understand
# what the GPL is, find it and read it BEFORE using this software.
# -------------------------------------------------------------------
# REQUIREMENTS:
# -------------
# 1. MySQL database
#
# 2. PERL 5 or higher
#
# 3. DBI Interface to MySQL database
#
# 4. A database created with SortMp3s.pl
#
# -------------------------------------------------------------------
# INSTRUCTIONS:
# -------------
# Set all of the values at the beginning of the script to match your
# setup. Run the script. Watch the magic! 
# -------------------------------------------------------------------
# DISCLAIMER:
# -----------
# I do not take any responsibility for anything that happens to your
# system or you personally for using this script. USE THIS SCRIPT AT
# YOUR OWN RISK. If you do not agree with this, do not use this 
# script.
#
# I do not provide any support for this script. I wrote it for my 
# personal use, and thought it would be useful for others. If you
# need a a custom solution, feel free to contact me and we can work
# out payment arrangements for custom development work. I am a very
# busy programmer, therefore I can not do free custom programming 
# work or help everyone set this script up.
#
# However, if you find bugs within this script, please send me your
# bugfixes and I will integrate them into a future release of the
# script.
# ------------------------------------------------------------------

use strict;
use DBI;

eval {
      # ------- Set up these values to match your system setup. -------
   
   # This is the hostname where your MySQL server resides. For most
   # setups, you can leave it as is.
   my $DB_HOST = '';

   # This is the port on $DB_HOST where your MySQL server resides.
   # For most setups, you can leave it as is.
   my $DB_PORT = '';

   # This is the database name to use to store the information. You
   # should create this database for use by this application. To 
   # create the tables required for the application, see the file
   # included in this application titled 'INSTALL'
   my $DB_NAME = 'MP3s';

   # This is the username to be used for connecting to $DB_NAME. 
   # This user must have full access to $DB_NAME
   my $DB_USER = 'MP3s';

   # This is the password for the user $DB_USER.
   my $DB_PASS = 'MP3s';

   # The type of list you want. Available types 
   # are:
   #
   #   xmms = XMMS Playlist
   #
   #   html = HTML Format for a web page.
   #
   my $REPORT_TYPE = 'xmms';

   # Make the database connection
   my $dsn = "DBI:mysql:database=$DB_NAME";
   $dsn .= ";host=$DB_HOST" if $DB_HOST;
   $dsn .= ";port=$DB_PORT" if $DB_PORT;
   my $dbh = DBI->connect( $dsn, $DB_USER, $DB_PASS );
   
   if ( ! $dbh ) {
      die;
   }

   # Perform the query.
   my $sql = qq[
      SELECT Genres.Name AS Genre,
             Artists.Name AS Artist,
             Albums.Name AS Album,
             Songs.Name,
             Songs.Year,
             Songs.TrackNum,
             Songs.FileSize,
             Songs.FileName,
             Songs.Comment
       FROM  Songs
       LEFT JOIN Genres ON Genres.id = Songs.Genre
       LEFT JOIN Artists ON Artists.id = Songs.Artist
       LEFT JOIN Albums ON Albums.id = Songs.Album
       ORDER BY Artists.Name, Albums.Name, Songs.TrackNum, Songs.Name]; 
   my $sth = $dbh->prepare($sql);
   $sth->execute();
   
   my $output = startTable() if $REPORT_TYPE ne 'xmms';
   while( my $song = $sth->fetchrow_hashref() ) {
      if ( $REPORT_TYPE eq 'xmms' ) {
         $output .= "$song->{FileName}\n";
      } else {
         $output .= tableRow( $song->{Genre},
                              $song->{Artist},
                              $song->{Album},
                              $song->{Name},
                              $song->{Year},
                              $song->{TrackNum},
                              $song->{FileSize},
                              $song->{FileName},
                              $song->{Comment} ) if $REPORT_TYPE ne 'xmms';
      }
      
   }
   $output .= endTable() if $REPORT_TYPE ne 'xmms';
   
   open(OUT, ">output.$REPORT_TYPE" );
   print OUT $output;
   close( OUT );

   sub startTable {
      return qq[
<html>
<head>
<title>MY MP3 LIST</title>
</head>
<body bgcolor="#FFFFFF">
<center>
<table border="1" cellpadding="3" cellspacing="0">
  <thead>
  <tr>
    <td bgcolor="#000066" colspan="5"><font face="arial, helvetica, lucida" size="4" color="#FFFFFF">MY MP3s</font></td>
  </tr>
  <tr>
    <th bgcolor="#A0A0FF" id="year"><font face="arial, helvetica, lucida" size="2">YEAR</font></th>
    <th bgcolor="#A0A0FF" id="genre"><font face="arial, helvetica, lucida" size="2">GENRE</font></th>
    <th bgcolor="#A0A0FF" id="name"><font face="arial, helvetica, lucida" size="2">ARTIST - TITLE</font></th>
    <th bgcolor="#A0A0FF" id="album"><font face="arial, helvetica, lucida" size="2">ALBUM - TRACK</font></th>
    <th bgcolor="#A0A0FF" id="size"><font face="arial, helvetica, lucida" size="2">SIZE</font></th>
  </tr>
  </thead>
  <tbody>];
   }
   
   sub tableRow {
      my ( $genre, $artist, $album, $name, $year, $track, $size, $filename, $comment ) = @_;
      $year = '&nbsp;' if $year eq 'UNKN' || $year < 1980;
      $track = '&nbsp;' if $track eq 'UN';
      $album = '&nbsp;' if $album eq 'UNKNOWN';
      $artist = '&nbsp;' if $artist eq 'UNKNOWN';
      $genre = '&nbsp;' if $genre eq 'UNKNOWN';
      if ( $artist ne '&nbsp;' ) {
         $name = "$artist - $name";
      }
      my $atrack = '&nbsp';
      if ( $album ne '&nbsp;' && $track ne '&nbsp;' ) {
         $atrack = "$album: TRACK $track";
      } elsif ( $album ne '&nbsp;' ) {
         $atrack = $album;
      }
      return qq[
  <tr>
    <td headers="year"><font face="arial, helvetica, lucida" size="2">$year</font></td>
    <td headers="genre"><font face="arial, helvetica, lucida" size="2">$genre</font></td>
    <td headers="name"><font face="arial, helvetica, lucida" size="2"><A HREF="$filename">$name</a></font></td>
    <td headers="album"><font face="arial, helvetica, lucida" size="2">$atrack</font></td>
    <td headers="size"><font face="arial, helvetica, lucida" size="2">$size</font></td>
  </tr>];
   }
   
   sub endTable {
      return qq[
  </tbody>
</table>
</center>
</body>
</html>];
   }

};
